@@ -13,7 +13,9 @@ class PyXploraApi:
 
         self.tokenExpiresAfter = 240
         self.maxRetries = 3
-        self.retryDelay = 3
+        self.retryDelay = 2
+
+        self.dtIssueToken = int(time.time()) - (self.tokenExpiresAfter * 1000)
 
         self.contacts = []
         self.alarms = []
@@ -30,7 +32,7 @@ class PyXploraApi:
         self.watch = None
         self.user = None
 
-    def __login(self, forceLogin=False):
+    def __login(self, forceLogin=False) -> dict:
         if not self.__isConnected() or self.__hasTokenExpired() or forceLogin:
             try:
                 self.__logoff()
@@ -59,17 +61,17 @@ class PyXploraApi:
                 self.__issueToken = None
         return self.__issueToken
 
-    def __isConnected(self):
+    def __isConnected(self) -> bool:
         return (self.__gqlHandler and self.__issueToken)
 
-    def __logoff(self):
+    def __logoff(self) -> None:
         self.__gqlHandler = None
         self.__issueToken = None
 
-    def __hasTokenExpired(self):
+    def __hasTokenExpired(self) -> bool:
         return ((int(time.time()) - self.dtIssueToken) > (self.tokenExpiresAfter * 1000))
 
-    def init(self, forceLogin=False):
+    def init(self, forceLogin=False) -> None:
         token = self.__login(forceLogin)
         if token:
             if ('user' in token):
@@ -82,7 +84,7 @@ class PyXploraApi:
         return VERSION
 
 ##### Contact Info #####
-    def getContacts(self):
+    def getContacts(self) -> list:
         retryCounter = 0
         dataOk = False
         contacts_raw = None
@@ -183,7 +185,7 @@ class PyXploraApi:
             raise Exception('Xplora API call finally failed with response: ')
         return self.alarms
 
-    def loadWatchLocation(self, withAsk=True):
+    def loadWatchLocation(self, withAsk=True) -> list:
         retryCounter = 0
         dataOk = False
         location_raw = None
@@ -193,22 +195,37 @@ class PyXploraApi:
             try:
                 if withAsk:
                     self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 location_raw = self.__gqlHandler.getWatchLastLocation(self.getWatchUserID())
                 if 'watchLastLocate' in location_raw:
-                    self.watch_location.append({
-                        'tm': datetime.fromtimestamp(location_raw['watchLastLocate']['tm']).strftime('%Y-%m-%d %H:%M:%S'),
-                        'lat': location_raw['watchLastLocate']['lat'],
-                        'lng': location_raw['watchLastLocate']['lng'],
-                        'rad': location_raw['watchLastLocate']['rad'],
-                        'poi': location_raw['watchLastLocate']['poi'],
-                        'city': location_raw['watchLastLocate']['city'],
-                        'province': location_raw['watchLastLocate']['province'],
-                        'country': location_raw['watchLastLocate']['country'],
-                    })
-                    self.watch_battery = location_raw['watchLastLocate']['battery']
-                    self.watch_charging = location_raw['watchLastLocate']['isCharging']
-                    self.watch_last_location = location_raw['watchLastLocate']
+                    if location_raw['watchLastLocate'] == None:
+                        self.watch_location.append({
+                            'tm': datetime.fromtimestamp(datetime.now().timestamp()).strftime('%Y-%m-%d %H:%M:%S'),
+                            'lat': 42.8491703,
+                            'lng': 33.5659302,
+                            'rad': 0,
+                            'poi': '',
+                            'city': '',
+                            'province': '',
+                            'country': '',
+                        })
+                        self.watch_battery = 0
+                        self.watch_charging = False
+                        self.watch_last_location = []
+                    else:
+                        self.watch_location.append({
+                            'tm': datetime.fromtimestamp(location_raw['watchLastLocate']['tm']).strftime('%Y-%m-%d %H:%M:%S'),
+                            'lat': location_raw['watchLastLocate']['lat'],
+                            'lng': location_raw['watchLastLocate']['lng'],
+                            'rad': location_raw['watchLastLocate']['rad'],
+                            'poi': location_raw['watchLastLocate']['poi'],
+                            'city': location_raw['watchLastLocate']['city'],
+                            'province': location_raw['watchLastLocate']['province'],
+                            'country': location_raw['watchLastLocate']['country'],
+                        })
+                        self.watch_battery = location_raw['watchLastLocate']['battery']
+                        self.watch_charging = location_raw['watchLastLocate']['isCharging']
+                        self.watch_last_location = location_raw['watchLastLocate']
             except Exception as error:
                 print(error)
             dataOk = self.watch_location
@@ -237,7 +254,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 ask_raw = self.askWatchLocate()
                 track_raw = self.trackWatchInterval()
                 if ask_raw or (track_raw != -1):
@@ -267,7 +284,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 chats_raw = self.__gqlHandler.chats(self.getWatchUserID())
                 if 'chats' in chats_raw:
                     if 'list' in chats_raw['chats']:
@@ -321,7 +338,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 safeZones_raw = self.__gqlHandler.safeZones(self.getWatchUserID())
                 if 'safeZones' in safeZones_raw:
                     for safeZone in safeZones_raw['safeZones']:
@@ -359,7 +376,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 sientTimes_raw = self.__gqlHandler.silentTimes(self.getWatchUserID())
                 if 'silentTimes' in sientTimes_raw:
                     for sientTime in sientTimes_raw['silentTimes']:
@@ -389,7 +406,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 enable_raw = self.__gqlHandler.setEnableSlientTime(silentId)
                 if 'setEnableSilentTime' in enable_raw:
                     _raw = enable_raw['setEnableSilentTime']
@@ -412,7 +429,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 disable_raw = self.__gqlHandler.setEnableSlientTime(silentId, NormalStatus.DISABLE.value)
                 if 'setEnableSilentTime' in disable_raw:
                     _raw = disable_raw['setEnableSilentTime']
@@ -446,7 +463,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 enable_raw = self.__gqlHandler.setEnableAlarmTime(alarmId)
                 if 'modifyAlarm' in enable_raw:
                     _raw = enable_raw['modifyAlarm']
@@ -469,7 +486,7 @@ class PyXploraApi:
             self.init()
             try:
                 self.askWatchLocate()
-                time.sleep(2)
+                time.sleep(self.retryDelay)
                 disable_raw = self.__gqlHandler.setEnableAlarmTime(alarmId, NormalStatus.DISABLE.value)
                 if 'modifyAlarm' in disable_raw:
                     _raw = disable_raw['modifyAlarm']
@@ -494,9 +511,9 @@ class PyXploraApi:
             res.append(self.setDisableAlarmTime(alarmTime['id']))
         return res
 
-    def sendText(self, text): # sender is login User
+    def sendText(self, text) -> bool: # sender is login User
         return self.__gqlHandler.sendText(self.getWatchUserID(), text)
-    def isAdmin(self):
+    def isAdmin(self) -> bool:
         for contact in self.getContacts():
             if (contact['id'] == self.getUserID()):
                 return True
